@@ -49,20 +49,23 @@ from datalad.distribution.dataset import require_dataset
 from datalad.distribution.dataset import EnsureDataset
 from datalad.distribution.dataset import datasetmethod
 
-from datalad.utils import assure_bytes
-from datalad.utils import assure_unicode
-from datalad.utils import chpwd
-from datalad.utils import get_dataset_root
-from datalad.utils import getpwd
-from datalad.utils import SequenceFormatter
-from datalad.utils import quote_cmdlinearg
+from datalad.utils import (
+    chpwd,
+    ensure_bytes,
+    ensure_unicode,
+    get_dataset_root,
+    getpwd,
+    join_cmdline,
+    quote_cmdlinearg,
+    SequenceFormatter,
+)
 
 lgr = logging.getLogger('datalad.core.local.run')
 
 
 def _format_cmd_shorty(cmd):
     """Get short string representation from a cmd argument list"""
-    cmd_shorty = (' '.join(cmd) if isinstance(cmd, list) else cmd)
+    cmd_shorty = (join_cmdline(cmd) if isinstance(cmd, list) else cmd)
     cmd_shorty = u'{}{}'.format(
         cmd_shorty[:40],
         '...' if len(cmd_shorty) > 40 else '')
@@ -120,7 +123,7 @@ class Run(Interface):
       Add a placeholder "name" with the value "joe"::
 
         % git config --file=.datalad/config datalad.run.substitutions.name joe
-        % datalad add -m "Configure name placeholder" .datalad/config
+        % datalad save -m "Configure name placeholder" .datalad/config
 
       Access the new placeholder in a command::
 
@@ -346,12 +349,15 @@ def prepare_inputs(dset_path, inputs, extra_inputs=None):
     for gp in gps:
         for res in _install_and_reglob(dset_path, gp):
             yield res
-        for res in get(dataset=dset_path, path=gp.expand(), on_failure="ignore"):
-            if _is_nonexistent_path(res):
-                # MIH why just a warning if given inputs are not valid?
-                lgr.warning("Input does not exist: %s", res["path"])
-            else:
-                yield res
+        if gp.misses:
+            ds = Dataset(dset_path)
+            for miss in gp.misses:
+                yield get_status_dict(
+                    action="run", ds=ds, status="error",
+                    message=("Input did not match existing file: %s",
+                             miss))
+        yield from get(dataset=dset_path, path=gp.expand_strict(),
+                       on_failure="ignore")
 
 
 def _unlock_or_remove(dset_path, paths):
@@ -396,7 +402,7 @@ def normalize_command(command):
     """Convert `command` to the string representation.
     """
     if isinstance(command, list):
-        command = list(map(assure_unicode, command))
+        command = list(map(ensure_unicode, command))
         if len(command) == 1 and command[0] != "--":
             # This is either a quoted compound shell command or a simple
             # one-item command. Pass it as is.
@@ -413,9 +419,9 @@ def normalize_command(command):
                 # Strip disambiguation marker. Note: "running from Python API"
                 # FIXME from below applies to this too.
                 command = command[1:]
-            command = " ".join(quote_cmdlinearg(c) for c in command)
+            command = join_cmdline(command)
     else:
-        command = assure_unicode(command)
+        command = ensure_unicode(command)
     return command
 
 
@@ -451,24 +457,16 @@ def format_command(dset, command, **kwds):
 
 
 def _execute_command(command, pwd, expected_exit=None):
-    from datalad.cmd import Runner
+    from datalad.cmd import WitlessRunner
 
     exc = None
     cmd_exitcode = None
-    runner = Runner(cwd=pwd)
+    runner = WitlessRunner(cwd=pwd)
     try:
         lgr.info("== Command start (output follows) =====")
         runner.run(
-            command,
-            # immediate output
-            log_online=True,
-            # not yet sure what we should do with the command output
-            # IMHO `run` itself should be very silent and let the command talk
-            log_stdout=False,
-            log_stderr=False,
-            expect_stderr=True,
-            expect_fail=True,
-            # TODO stdin
+            # command is always a string
+            command
         )
     except CommandError as e:
         # strip our own info from the exception. The original command output
@@ -495,8 +493,7 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
                 rerun_info=None,
                 extra_inputs=None,
                 rerun_outputs=None,
-                inject=False,
-                saver=None):
+                inject=False):
     """Run `cmd` in `dataset` and record the results.
 
     `Run.__call__` is a simple wrapper over this function. Aside from backward
@@ -525,8 +522,6 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
         preparation and command execution. In this mode, the caller is
         responsible for ensuring that the state of the working tree is
         appropriate for recording the command's results.
-    saver : None
-        This is obsolete and ignored. It will be removed in a later release.
 
     Yields
     ------
@@ -535,10 +530,6 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
     if not cmd:
         lgr.warning("No command given")
         return
-    if saver:
-        warnings.warn("`saver` argument is ignored "
-                      "and will be removed in a future release",
-                      DeprecationWarning)
 
     rel_pwd = rerun_info.get('pwd') if rerun_info else None
     if rel_pwd and dataset:
@@ -550,7 +541,7 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
 
     ds = require_dataset(
         dataset, check_installed=True,
-        purpose='tracking outcomes of a command')
+        purpose='track command outcomes')
     ds_path = ds.path
 
     lgr.debug('tracking command output underneath %s', ds)
@@ -591,7 +582,7 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
             if outputs:
                 for res in _install_and_reglob(ds_path, outputs):
                     yield res
-                for res in _unlock_or_remove(ds_path, outputs.expand()):
+                for res in _unlock_or_remove(ds_path, outputs.expand_strict()):
                     yield res
 
             if rerun_outputs is not None:
@@ -628,6 +619,13 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
             expected_exit=rerun_info.get("exit", 0) if rerun_info else None)
 
 
+    # Re-glob to capture any new outputs.
+    #
+    # TODO: If a warning or error is desired when an --output pattern doesn't
+    # have a match, this would be the spot to do it.
+    if explicit or expand in ["outputs", "both"]:
+        outputs.expand(refresh=True)
+
     # amend commit message with `run` info:
     # - pwd if inside the dataset
     # - the command itself
@@ -652,11 +650,6 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
 
     if sidecar is None:
         use_sidecar = ds.config.get('datalad.run.record-sidecar', default=False)
-        # If ConfigManager gets the ability to say "return single value",
-        # update this code to use that.
-        if isinstance(use_sidecar, tuple):
-            # Use same precedence as 'git config'.
-            use_sidecar = use_sidecar[-1]
         use_sidecar = anything2bool(use_sidecar)
     else:
         use_sidecar = sidecar
@@ -685,7 +678,7 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
         message if message is not None else _format_cmd_shorty(cmd_expanded),
         '"{}"'.format(record_id) if use_sidecar else record)
 
-    outputs_to_save = outputs.expand() if explicit else None
+    outputs_to_save = outputs.expand_strict() if explicit else None
     if outputs_to_save is not None and use_sidecar:
         outputs_to_save.append(record_path)
     do_save = outputs_to_save is None or outputs_to_save
@@ -694,7 +687,7 @@ def run_command(cmd, dataset=None, inputs=None, outputs=None, expand=None,
             repo = ds.repo
             msg_path = relpath(opj(str(repo.dot_git), "COMMIT_EDITMSG"))
             with open(msg_path, "wb") as ofh:
-                ofh.write(assure_bytes(msg))
+                ofh.write(ensure_bytes(msg))
             lgr.info("The command had a non-zero exit code. "
                      "If this is expected, you can save the changes with "
                      "'datalad save -d . -r -F %s'",
