@@ -44,7 +44,11 @@ from datalad.tests.utils import (
     assert_repo_status,
     assert_result_count,
     assert_message,
+    DEFAULT_REMOTE,
     serve_path_via_http,
+    skip_if_adjusted_branch,
+    skip_ssh,
+    skip_if_on_windows,
     slow,
     known_failure_windows,
     known_failure_githubci_win,
@@ -52,6 +56,7 @@ from datalad.tests.utils import (
 from datalad.utils import (
     with_pathsep,
     chpwd,
+    Path,
     rmtree,
 )
 from ..dataset import Dataset
@@ -99,20 +104,20 @@ def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
     # own location default remote for current branch
     clone_subpath = str(clone.pathobj / 'sub')
     eq_(f(clone, dict(path=clone_subpath, parentds=clone.path)),
-        [dict(cost=500, name='origin', url=ds_subpath)])
+        [dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath)])
     eq_(f(clone, dict(path=clone_subpath, parentds=clone.path, gitmodule_url=sshurl)),
-        [dict(cost=500, name='origin', url=ds_subpath),
-         dict(cost=600, name='origin', url=sshurl)])
+        [dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
+         dict(cost=600, name=DEFAULT_REMOTE, url=sshurl)])
     eq_(f(clone, dict(path=clone_subpath, parentds=clone.path, gitmodule_url=httpurl)),
-        [dict(cost=500, name='origin', url=ds_subpath),
-         dict(cost=600, name='origin', url=httpurl)])
+        [dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
+         dict(cost=600, name=DEFAULT_REMOTE, url=httpurl)])
 
     # make sure it does meaningful things in an actual clone with an actual
     # record of a subdataset
     clone_subpath = str(clone.pathobj / 'sub')
     eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
         [
-            dict(cost=500, name='origin', url=ds_subpath),
+            dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
     ])
 
     # check that a configured remote WITHOUT the desired submodule commit
@@ -121,7 +126,7 @@ def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
                    result_renderer='disabled')
     eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
         [
-            dict(cost=500, name='origin', url=ds_subpath),
+            dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
     ])
     # inject a source URL config, should alter the result accordingly
     with patch.dict(
@@ -129,7 +134,7 @@ def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
             {'DATALAD_GET_SUBDATASET__SOURCE__CANDIDATE__BANG': 'youredead'}):
         eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
             [
-                dict(cost=500, name='origin', url=ds_subpath),
+                dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
                 dict(cost=700, name='bang', url='youredead', from_config=True),
         ])
     # we can alter the cost by given the name a two-digit prefix
@@ -139,7 +144,7 @@ def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
         eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
             [
                 dict(cost=400, name='bang', url='youredead', from_config=True),
-                dict(cost=500, name='origin', url=ds_subpath),
+                dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
         ])
     # verify template instantiation works
     with patch.dict(
@@ -147,7 +152,7 @@ def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
             {'DATALAD_GET_SUBDATASET__SOURCE__CANDIDATE__BANG': 'pre-{id}-post'}):
         eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
             [
-                dict(cost=500, name='origin', url=ds_subpath),
+                dict(cost=500, name=DEFAULT_REMOTE, url=ds_subpath),
                 dict(cost=700, name='bang', url='pre-{}-post'.format(sub.id),
                      from_config=True),
         ])
@@ -637,3 +642,57 @@ def test_gh3356(src, path):
     assert_result_count(
         clone.status(recursive=True, annex='all', report_filetype='eval'), 2,
         action='status', has_content=True)
+
+
+# The setup here probably breaks down with adjusted branches.
+@skip_if_adjusted_branch
+@slow  # ~12s
+@skip_if_on_windows
+@skip_ssh
+@with_tempfile(mkdir=True)
+def test_get_subdataset_direct_fetch(path):
+    path = Path(path)
+    origin = Dataset(path / "origin").create()
+    for sub in ["s0", "s1"]:
+        sds = origin.create(origin.pathobj / sub)
+        sds.repo.commit(msg="another commit", options=["--allow-empty"])
+    origin.save()
+    s0 = Dataset(origin.pathobj / "s0")
+    s1 = Dataset(origin.pathobj / "s1")
+    # Abandon the recorded commit so that it needs to be brought down by a
+    # direct fetch.
+    s0.repo.call_git(["reset", "--hard", "HEAD~"])
+    s1.repo.call_git(["reset", "--hard", "HEAD~"])
+
+    # Tweak the configuration of s0 to make the direct fetch fail.
+    # Disallow direct oid fetch (default).
+    s0.repo.config.set("uploadpack.allowAnySHA1InWant", "false",
+                       where="local")
+    # Configure the fetcher to avoid v2, which allows fetching unadvertised
+    # objects regardless of the value of uploadpack.allowAnySHA1InWant.
+    s0.repo.config.set("protocol.version", "0", where="local")
+
+    # Configure s1 to succeed with direct fetch.
+    s1.repo.config.set("uploadpack.allowAnySHA1InWant", "true",
+                       where="local")
+
+    clone = install(
+        str(path / "clone"),
+        source="ssh://datalad-test:" + origin.repo.pathobj.as_posix())
+
+    res = clone.get(["s0", "s1"], on_failure="ignore")
+    assert_result_count(res, 1,
+                        action="install", type="dataset", status="error")
+    assert_result_count(res, 1,
+                        action="install", type="dataset", status="ok")
+
+
+@with_tempfile()
+def test_get_relays_command_errors(path):
+    ds = Dataset(path).create()
+    (ds.pathobj / "foo").write_text("foo")
+    ds.save()
+    ds.drop("foo", check=False)
+    assert_result_count(
+        ds.get("foo", on_failure="ignore", result_renderer=None),
+        1, action="get", type="file", status="error")
